"""Basic plotting tools."""
import os

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.transforms import Bbox
import scipy.constants as c

from plasmatk import parameters as p

def plot_mag_regimes(
        couple_bounds=(1e-3, 1e2), mag_bounds=(1e-4,1e3),
        add_regions=True, region_txt=True, colors=None,
        line_params=None, boundary_labels=False,
    ):
    """Create a plot of the 4 magnetic/coupling regimes
    and the boundaries between them.
    
    Args:
        couple_bounds (tuple[float]): The coupling constant
            axis limits.
        mag_bounds (tuple[float]): The magnetization parameter
            axis limits.
        add_regions (bool): 

    """
    fig, ax = plt.subplots()
    ax.set_xlim(*couple_bounds)
    ax.set_ylim(*mag_bounds)
    if line_params is None:
        line_params = {"c": "k"}
    # Number of sample
    ns = 1000
    log_cb = (np.log10(couple_bounds[0]),
              np.log10(couple_bounds[1]))
    Gamma = np.logspace(*log_cb, ns)
    # Unmagnitized boundary
    _make_boundary(ax, p.lcol_b, Gamma, 1e-2, r"$r_c = \lambda_\text{col}$",
       {"xytext": (0,1)}, region_txt, **line_params)
    # Weakly magnetized boundary
    _make_boundary(ax, p.lD_b, Gamma, 1e-2, r"$r_c = \lambda_\text{D}$",
       {"xytext": (0,-1)}, region_txt, **line_params)
    _make_boundary(ax, p.r_L, Gamma, .03, r"$r_c = r_\text{L}$",
       {"xytext": (0,-1)}, region_txt, **line_params)
    _make_boundary(ax, p.r_L, Gamma, .9, r"$r_c = a$",
       {"xytext": (0,0)}, region_txt, **line_params)
    
    # MITL condition boundaries
    T_b = (1e2, 1e5)
    n_b = (1e13, 1e17)
    B_b = (20, 200)
    G_b = (p.Gamma_e(T_b[1], n_b[0]),
           p.Gamma_e(T_b[0], n_b[1]))
    Be_b = (p.Beta_e(B_b[0], n_b[1]),
           p.Beta_e(B_b[1], n_b[0]))
    _add_region(ax, G_b, Be_b, label=r"z-machine\nMITL")

    ax.set_xscale("log")
    ax.set_yscale("log")
    ax.set_title("Magnetization-Coupling Phase Space")
    ax.set_xlabel(r"Coupling Strength ($\Gamma$)")
    ax.set_ylabel(r"Magnetization ($\omega_c/\omega_p$)")
    fig.tight_layout()
    return fig, ax

def _make_boundary(ax, f, G, sp, text, tp, rt, **line_params):
    B = np.vectorize(f)(G)
    ax.plot(G, B, **line_params)
    if rt:
        # Find rotation based off dB/dG
        pct = .08
        xl, yl = ax.get_xlim(), ax.get_ylim()
        log_rng = np.log10(xl[1]/xl[0])
        log_yrng = np.log10(yl[1]/yl[0])
        ar = log_yrng / log_rng
        bbox = ax.get_window_extent().transformed(
               ax.figure.dpi_scale_trans.inverted())
        dar = bbox.width / bbox.height
        x1, x2 = np.log10(sp), np.log10(sp) + pct*log_rng
        y1, y2 = np.log10(f(sp)), np.log10(f(10**x2))
        ax.scatter([10**x1, 10**x2], [10**y1, 10**y2])
        rot = 180/np.pi*np.arctan((y2-y1)/(x2-x1)) / ar / dar
        ax.annotate(text, (sp, f(sp)), fontsize=12,
            rotation=rot*.9, rotation_mode="anchor",
            textcoords="offset fontsize", **tp)

def _add_region(ax, x, y, label=None):
    """Mark out a region of the phase space."""
    # Draw rectangle
    rect = plt.Rectangle(
        (x[0], y[0]), x[1]-x[0], y[1]-y[0],
        ls="--", fill=False
        )
    ax.add_patch(rect)

def plot_mag_regimes_nT():
    """Create a plot of the 4 magnetic/coupling regimes
    in density-temperature space."""
    # Change
    pass

def plot_mag_regimes_nTB():
    pass



import numpy as np
import pandas as pd
import scipy.constants as c

e = c.e
m_e = c.m_e
epsilon_0 = c.epsilon_0

class Parameters(object):
    r"""An enumeration of input plasma parameters.

    Args:
        T (array-like): Temperature (eV).
        n_e (array-like): Electron density :math:`\text{m}^{-3}`.
        B (array-like): Magnetic field strength (T).
    """
    def __init__(T=None, n_e=None, B=None):
        self.T = T
        self.n_e = n_e
        self.B = B

def Gamma_e(T, n_e):
    """Return the coupling constant.

    Args:
        p (Parameters): Termperature and density required
        T (float): The temperature in eV.
        n_e (float): The electron number density in cm^-3.
    Returns:
        The electron coupling parameter.
    """
    # Convert to meters
    n = n_e * 100**3
    a_e = (3/(4*np.pi*n))**(1/3)
    return c.e**2/(
        a_e*4*np.pi*c.epsilon_0*c.e*T)

def Beta_e(B, n_e):
    r"""Return the magnetization parameter
    :math:`B \equiv \frac{\omega_{ce}}{\omega_{pe}}`.

    Args:
        B (float): The magnetic field in Tesla.
        n_e (float): The electron number density in cm^-3.
    """
    n = n_e * 100**3
    return B*np.sqrt(c.epsilon_0 / (c.m_e * n))

def Theta(Gamma):
    """The generalized coulomn logarithm.

    Args:
        Gamma: The electron coupling parameter
    """
    return 0.65 * np.log(1 + 2.15/(np.sqrt(3)*Gamma**(3/2)))

def v_the(n, T, B):
    r"""Return the thermal speed. Defined as
    :math:`v_\mathrm{th} = \sqrt{\frac{k_\mathrm{B}T}{m_\mathrm{e}}}`.

    Args:
        1
    """
    # Convert from keV to Joules
    E = T * 1000 * e
    return np.sqrt(E/m_e)

def l_De(n, T, B):
    """The electron Debye length."""
    # Convert from keV to Joules
    E = T * 1000 * e
    return np.sqrt(epsilon_0 * E / (n*e**2))

def N_De(n, T, B):
    return n*l_De(n, T, B)**3

def wpe(n, T, B):
    return np.sqrt(c.e**2 * n / (epsilon_0 * m_e) )

def wce(n, T, B):
    """The electron cyclofrequency."""
    return e*B/m_e

def lcol_b(Gamma):
    """The boundary condition r_c = l_col"""
    return Theta(Gamma)*0.32*Gamma**(3/2)

def lD_b(Gamma):
    """The boundary condition r_c = l_D."""
    Aa = np.sqrt(3*.55/2)
    return max(min(1., Aa*np.sqrt(2/(3*Gamma))), lcol_b(Gamma))

def r_L(Gamma):
    """The boundary condition r_c = r_L."""
    Aa = np.sqrt(3*.55/2)
    AL = np.sqrt(6*.55**3)
    return max(AL*np.sqrt(1.0/(6*Gamma**3)),
       Aa*np.sqrt(2/(3*Gamma)), lcol_b(Gamma))



from importlib.metadata import version
from importlib_resources import files
import os
from os.path import join as pjoin

__version__ = version(__name__)

src_path = files(__name__).joinpath("cpp")

from . import parameters
from . import plotting


